import java.util.*;

public class Solver {

    Parser parser = new Parser();

    List<Integer> indexChemin; // Liste pour stocker les indices des lieux dans le chemin le plus court
    List<Lieu> chemin = new ArrayList<>();


    Solver(int indexDepart, int indexArrive){

        // Lieux de la carte
        List<Lieu> lieux = parser.lieux;

        if (indexDepart < 0 || indexDepart >= lieux.size() || indexArrive < 0 || indexArrive >= lieux.size()) {
            throw new IllegalArgumentException("Lieux de départ ou d'arrivée invalides.");
        }

        // Construction de la matrice de distance

        int n = lieux.size();
        double[][] matriceDistance = new double[n][n];

        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                if (i == j) {
                    matriceDistance[i][j] = 0;
                } else {
                    // Calculer la distance entre les villes i et j
                    for (Route route: parser.routes){
                        if (isEqual(lieux.get(i),route.depart) && isEqual(lieux.get(j),route.arrive)){
                            matriceDistance[i][j] = route.distanceRoute;
                        }
                    }
                }
            }
        }
        // Appel à l'algorithme de Dijkstra pour trouver le chemin le plus court
        this.indexChemin = dijkstra(matriceDistance, indexDepart, indexArrive);

        //Construction de la liste des lieux dans le chemin le plus court
        indexChemin.forEach(i -> this.chemin.add(lieux.get(i)));
    }

    public boolean isEqual(Lieu l1, Lieu l2){
        // Méthode pour vérifier si deux lieux sont égaux
        return l1.pos == l2.pos && Objects.equals(l1.ville, l2.ville) && Objects.equals(l1.nomLieu, l2.nomLieu) && l1.typeLieu == l2.typeLieu;
    }

    public static List<Integer> dijkstra(double[][] matriceDistance, int depart, int arrive) {
        int n = matriceDistance.length;
        double[] dist = new double[n];
        int[] prev = new int[n];
        boolean[] visited = new boolean[n];

        // Initialisation des distances à l'infini
        for (int i = 0; i < n; i++) {
            dist[i] = Double.POSITIVE_INFINITY;
            prev[i] = -1;
        }
        dist[depart] = 0;

        for (int i = 0; i < n - 1; i++) {
            // Trouver le sommet non visité avec la distance minimale
            int u = -1;
            for (int j = 0; j < n; j++) {
                if (!visited[j] && (u == -1 || dist[j] < dist[u])) {
                    u = j;
                }
            }

            // Marquer le sommet comme visité
            visited[u] = true;

            // Mettre à jour les distances des sommets adjacents
            for (int v = 0; v < n; v++) {
                if (!visited[v] && matriceDistance[u][v] != 0) {
                    double newDist = dist[u] + matriceDistance[u][v];
                    if (newDist < dist[v]) {
                        dist[v] = newDist;
                        prev[v] = u;
                    }
                }
            }
        }

        // Reconstruire le chemin le plus court
        List<Integer> chemin = new ArrayList<>();
        int u = arrive;
        while (u != -1) {
            chemin.add(u);
            u = prev[u];
        }
        Collections.reverse(chemin);

        return chemin;
    }

    public static void main(String[] args) {
        Parser parser = new Parser();
        List<Lieu> lieux = parser.lieux;

        Scanner sc = new Scanner(System.in);
        lieux.forEach(lieu -> System.out.println(lieu.toString()));

        System.out.println("Veuillez choisir votre lieu de départ parmi la liste ci dessus :");
        String debut = sc.nextLine();
        String[] elements = debut.split(" ");

        System.out.println("Veuillez choisir votre lieu d'arrivée parmi la liste ci dessus :");
        String fin = sc.nextLine();
        String[] elements2 = fin.split(" ");


        int indexDepart = -1;
        int indexArrive = -1;
        int tmpDep = 0;
        for (Lieu lieu : lieux) {
            if (elements.length>2) {
                if (Objects.equals(elements[0], lieu.nomLieu) && Objects.equals(elements[1]+' '+elements[2], lieu.ville)) {
                    indexDepart = tmpDep;
                }
            }else{
                if (Objects.equals(elements[0], lieu.nomLieu) && Objects.equals(elements[1], lieu.ville)) {
                    indexDepart = tmpDep;
                }
            }
            tmpDep++;
        }
        int tmpArr = 0;
        for (Lieu lieu : lieux) {
            if (elements2.length>2) {
                if (Objects.equals(elements2[0], lieu.nomLieu) && Objects.equals(elements2[1]+' '+elements2[2], lieu.ville)) {
                    indexArrive = tmpArr;
                }
            }else{
                if (Objects.equals(elements2[0], lieu.nomLieu) && Objects.equals(elements2[1], lieu.ville)) {
                    indexArrive = tmpArr;
                }
            }
            tmpArr++;
        }
        if (indexDepart == -1 || indexArrive == -1) {
            System.out.println("L'un des lieux saisis est incorrect. Veuillez vérifier l'orthographe.");
            return;
        }
        Solver solver = new Solver(indexDepart, indexArrive);
        solver.parser.routes.forEach(System.out::println);
        System.out.println("---chemin----");
        System.out.println(solver.chemin);
        solver.chemin.forEach(lieu -> System.out.println(lieu.nomLieu + " " +  lieu.ville));
    }
}

